/* sevenalpha.m by Ariszlo <ariszlo@gmail.com> 2023-07-27

Switch between 7 different layouts and show the visualization container
through a transparent region of the main window. */

// This is needed for every MAKI script:
#include "lib/std.mi"

// Scripts which save settings need this, too:
#include "lib/config.mi"

// List your *global* variables here to be used throughout this script.
// All of these global variables take their values from xml objects identified
// by their id's. Read through the System.onScriptLoaded() function to see
// how the values are assigned to the variables.
Global Container mainContainer;

Global Layout mainNormal;
Global Layout mainNormalCover;
Global Layout mainNormalVis;
Global Layout mainCompact;
Global Layout mainCompactCover;
Global Layout mainCompactVis;
Global Layout mainStick;

Global Button normalSwitch;
Global Button normalCoverSwitch;
Global Button normalVisSwitch;
Global Button compactSwitch;
Global Button compactCoverSwitch;
Global Button compactVisSwitch;
Global Button stickSwitch;

Global Layer seethrough;
Global Layer cseethrough;

Global String cthrux, cthruy, cthruw, cthruh;
Global String ccthrux, ccthruy, ccthruw, ccthruh;

Global Int cthruleft, cthrutop, cthruwidth, cthruheight;
Global Int ccthruleft, ccthrutop, ccthruwidth, ccthruheight;

Global Double normalVisScale, compactVisScale;

Global String visx, visy, visw, vish, cvisx, cvisy, cvisw, cvish;

Global Container visContainer;
Global Layout visNormal, visCompact;

// Create a hidden config item with a config attribute to save
// the id of the main window's last used layout:
Global ConfigItem storeConfig;
Global ConfigAttribute storeAttrib;

// List your functions:
Function fitVis();
Function switchPopup();

// Switching from one vis layout to another is buggy.
// So create separate funtions for normalVis and compactVis:
Function normalVisPopup();
Function compactVisPopup();

// Do this when the script is loaded:
System.onScriptLoaded() {
  // Find the player window
  mainContainer = System.getContainer("main");

  // Find its layouts:
  mainNormal = mainContainer.getLayout("normal");
  mainNormalCover = mainContainer.getLayout("normalcover");
  mainNormalVis = mainContainer.getLayout("normalvis");
  mainCompact = mainContainer.getLayout("compact");
  mainCompactCover = mainContainer.getLayout("compactcover");
  mainCompactVis = mainContainer.getLayout("compactvis");
  mainStick = mainContainer.getLayout("stick");

  // Find the switch button in each layout:
  normalSwitch = mainNormal.findObject("Switch Layouts");
  normalCoverSwitch = mainNormalCover.findObject("Switch Layouts");
  normalVisSwitch = mainNormalVis.findObject("Switch Layouts");
  compactSwitch = mainCompact.findObject("Switch Layouts");
  compactCoverSwitch = mainCompactCover.findObject("Switch Layouts");
  compactVisSwitch = mainCompactVis.findObject("Switch Layouts");
  stickSwitch = mainStick.findObject("Switch Layouts");

  // Find the transparent layers in mainNormalVis & mainCompactVis:
  seethrough = mainNormalVis.findObject("seethrough");
  cseethrough = mainCompactVis.findObject("seethrough");

  // Get the position and size of the transparent layer in mainNormalVis
  // (cthru in the names of these variables is an abbreviation for seethrough):
  cthrux = seethrough.getXmlParam("x");
  cthruy = seethrough.getXmlParam("y");
  cthruw = seethrough.getXmlParam("w");
  cthruh = seethrough.getXmlParam("h");

  // Get the position and size of the transparent layer in mainCompactVis
  // (the initial c in the names of these variables stands for compact):
  ccthrux = cseethrough.getXmlParam("x");
  ccthruy = cseethrough.getXmlParam("y");
  ccthruw = cseethrough.getXmlParam("w");
  ccthruh = cseethrough.getXmlParam("h");

  // Turn the strings into integers:
  cthruleft = StringToInteger(cthrux);
  cthrutop = StringToInteger(cthruy);
  cthruwidth = StringToInteger(cthruw);
  cthruheight = StringToInteger(cthruh);

  ccthruleft = StringToInteger(ccthrux);
  ccthrutop = StringToInteger(ccthruy);
  ccthruwidth = StringToInteger(ccthruw);
  ccthruheight = StringToInteger(ccthruh);

  // Here comes the visualization container with its two layouts:
  visContainer = System.getContainer("viscontainer");
  visNormal = visContainer.getLayout("normal");
  visCompact = visContainer.getLayout("compact");

  // Generate a unique GUID for your config item:
  storeConfig = Config.newItem("Hidden","{7FF4CAF0-A66D-4C48-AF9D-D30440961FEA}");
  // Define the default layout for the main window:
  storeAttrib = storeConfig.newAttribute("storeAttrib","normal");
  // Read the id of the last used layout:
  storeAttrib.getData();

  // Execute the fitVis() funtion:
  fitVis();
}

// Fit the visualization window into the transparent region of the main window:
fitVis() {
  if (storeAttrib.getData() == "normalvis") {
    // Find out if the player is scaled and if so, how much:
    normalVisScale = mainNormalVis.getScale();
    // Calculate the position and size of the visualization window:
    visx = integerToString(mainNormalVis.getLeft() + (cthruleft * normalVisScale));
    visy = integerToString(mainNormalVis.getTop() + (cthrutop * normalVisScale));
    visw = integerToString(cthruwidth * normalVisScale);
    vish = integerToString(cthruheight * normalVisScale);
    // Set the position and size of the visualization window:
    visNormal.setXmlParam("x", visx);
    visNormal.setXmlParam("y", visy);
    visNormal.setXmlParam("w", visw);
    visNormal.setXmlParam("h", vish);
    // Show visualization:
    visNormal.show();
    visCompact.hide();
  }
  if (storeAttrib.getData() == "compactvis") {
    // Find out if the player is scaled and if so, how much:
    compactVisScale = mainCompactVis.getScale();
    // Calculate the position and size of the visualization window:
    cvisx = integerToString(mainCompactVis.getLeft() + (ccthruleft * compactVisScale));
    cvisy = integerToString(mainCompactVis.getTop() + (ccthrutop * compactVisScale));
    cvisw = integerToString(ccthruwidth * compactVisScale);
    cvish = integerToString(ccthruheight * compactVisScale);
    // Set the position and size of the visualization window:
    visCompact.setXmlParam("x", cvisx);
    visCompact.setXmlParam("y", cvisy);
    visCompact.setXmlParam("w", cvisw);
    visCompact.setXmlParam("h", cvish);
    // Show visualization:
    visNormal.hide();
    visCompact.show();
  }
  else {
    // Hide visualization:
    visContainer.hide();
  }
}

// Fix other containers sliding between main and vis:
mainNormalVis.onLeftButtonDown(int x, int y) {
  visNormal.setFocus();
}
mainCompactVis.onLeftButtonDown(int x, int y) {
  visCompact.setFocus();
}

// Move and scale the visualization container along with the main window:
mainNormalVis.onMove() {
  Double normalVisScale;
  normalVisScale = mainNormalVis.getScale();
  visx = integerToString(mainNormalVis.getLeft() + (cthruleft * normalVisScale));
  visy = integerToString(mainNormalVis.getTop() + (cthrutop * normalVisScale));
  visNormal.setXmlParam("x", visx);
  visNormal.setXmlParam("y", visy);
}

mainNormalVis.onScale(Double normalVisScale) {
  visx = integerToString(mainNormalVis.getLeft() + (cthruleft * normalVisScale));
  visy = integerToString(mainNormalVis.getTop() + (cthrutop * normalVisScale));
  visw = integerToString(cthruwidth * normalVisScale);
  vish = integerToString(cthruheight * normalVisScale);
  visNormal.setXmlParam("x", visx);
  visNormal.setXmlParam("y", visy);
  visNormal.setXmlParam("w", visw);
  visNormal.setXmlParam("h", vish);
}

mainCompactVis.onMove() {
  Double compactVisScale;
  compactVisScale = mainCompactVis.getScale();
  cvisx = integerToString(mainCompactVis.getLeft() + (ccthruleft * compactVisScale));
  cvisy = integerToString(mainCompactVis.getTop() + (ccthrutop * compactVisScale));
  visCompact.setXmlParam("x", cvisx);
  visCompact.setXmlParam("y", cvisy);
}

mainCompactVis.onScale(Double compactVisScale) {
  cvisx = integerToString(mainCompactVis.getLeft() + (ccthruleft * compactVisScale));
  cvisy = integerToString(mainCompactVis.getTop() + (ccthrutop * compactVisScale));
  cvisw = integerToString(ccthruwidth * compactVisScale);
  cvish = integerToString(ccthruheight * compactVisScale);
  visCompact.setXmlParam("x", cvisx);
  visCompact.setXmlParam("y", cvisy);
  visCompact.setXmlParam("w", cvisw);
  visCompact.setXmlParam("h", cvish);
}

// Pop up the Switch Layouts menu when the Switch Layouts button is clicked:
normalSwitch.onLeftClick() { switchPopup(); }
normalCoverSwitch.onLeftClick() { switchPopup(); }
compactSwitch.onLeftClick() { switchPopup(); }
compactCoverSwitch.onLeftClick() { switchPopup(); }
stickSwitch.onLeftClick() { switchPopup(); }

// Pop up shorter menus for normalVis and compactVis:
normalVisSwitch.onLeftClick() { normalVisPopup(); }
compactVisSwitch.onLeftClick() { compactVisPopup(); }

// The script receives your input:
normalSwitch.onLeftButtonUp(Int intPosX, Int intPosY) { complete;}
normalCoverSwitch.onLeftButtonUp(Int intPosX, Int intPosY) { complete;}
normalVisSwitch.onLeftButtonUp(Int intPosX, Int intPosY) { complete;}
compactSwitch.onLeftButtonUp(Int intPosX, Int intPosY) { complete;}
compactCoverSwitch.onLeftButtonUp(Int intPosX, Int intPosY) { complete;}
compactVisSwitch.onLeftButtonUp(Int intPosX, Int intPosY) { complete;}
stickSwitch.onLeftButtonUp(Int intPosX, Int intPosY) { complete;}

// Here comes the popup menu:
switchPopup() {
  // Create a *local* popup menu variable and name it switchMenu:
  PopupMenu switchMenu;
  // Define switchMenu as a new popup menu:
  switchMenu = new PopupMenu;

  // List what should appear in the menu:
  switchMenu.addCommand("Normal ", 1, (mainNormal.isVisible()), 0);
  switchMenu.addCommand("Normal Cover Art ", 2, (mainNormalCover.isVisible()), 0);
  switchMenu.addCommand("Normal Visualization ", 3, (mainNormalVis.isVisible()), 0);
  switchMenu.addCommand("Compact ", 4, (mainCompact.isVisible()), 0);
  switchMenu.addCommand("Compact Cover Art ", 5, (mainCompactCover.isVisible()), 0);
  switchMenu.addCommand("Compact Visualization ", 6, (mainCompactVis.isVisible()), 0);
  switchMenu.addCommand("Stick ", 7, (mainStick.isVisible()), 0);

  // Which click should trigger what action?
  if (switchMenu == Null) { return; }
  // Create a local integer variable which takes its value from the number
  // of the clicked menu item:
  Int switchNumber = switchMenu.popAtMouse();
  // Only do anything the value of switchNumber is greater than 0:
  if (switchNumber > 0) {
    if (switchNumber == 1) {
      mainContainer.switchToLayout("normal");
      storeAttrib.setData("normal");
      fitVis();
    }
    if (switchNumber == 2) {
      mainContainer.switchToLayout("normalcover");
      storeAttrib.setData("normalcover");
      fitVis();
    }
    if (switchNumber == 3) {
      mainContainer.switchToLayout("normalvis");
      visContainer.switchToLayout("normal");
      storeAttrib.setData("normalvis");
      fitVis();
    }
    if (switchNumber == 4) {
      mainContainer.switchToLayout("compact");
      storeAttrib.setData("compact");
      fitVis();
    }
    if (switchNumber == 5) {
      mainContainer.switchToLayout("compactcover");
      storeAttrib.setData("compactcover");
      fitVis();
    }
    if (switchNumber == 6) {
      mainContainer.switchToLayout("compactvis");
      visContainer.switchToLayout("compact");
      storeAttrib.setData("compactvis");
      fitVis();
    }
    if (switchNumber == 7) {
      mainContainer.switchToLayout("stick");
      storeAttrib.setData("stick");
      fitVis();
    }
  }
  complete;

  delete switchMenu;
}

// Don't switch from normalVis to compactVis:
normalVisPopup() {
  PopupMenu normalVisMenu;
  normalVisMenu = new PopupMenu;

  normalVisMenu.addCommand("Normal ", 1, (mainNormal.isVisible()), 0);
  normalVisMenu.addCommand("Normal Cover Art ", 2, (mainNormalCover.isVisible()), 0);
  normalVisMenu.addCommand("Normal Visualization ", 3, (mainNormalVis.isVisible()), 0);
  normalVisMenu.addCommand("Compact ", 4, (mainCompact.isVisible()), 0);
  normalVisMenu.addCommand("Compact Cover Art ", 5, (mainCompactCover.isVisible()), 0);
  normalVisMenu.addCommand("Stick ", 6, (mainStick.isVisible()), 0);

  if (normalVisMenu == Null) { return; }
  Int switchNumber = normalVisMenu.popAtMouse();
  if (switchNumber > 0) {
    if (switchNumber == 1) {
      mainContainer.switchToLayout("normal");
      storeAttrib.setData("normal");
      fitVis();
    }
    if (switchNumber == 2) {
      mainContainer.switchToLayout("normalcover");
      storeAttrib.setData("normalcover");
      fitVis();
    }
    if (switchNumber == 3) {
      mainContainer.switchToLayout("normalvis");
      visContainer.switchToLayout("normal");
      storeAttrib.setData("normalvis");
      fitVis();
    }
    if (switchNumber == 4) {
      mainContainer.switchToLayout("compact");
      storeAttrib.setData("compact");
      fitVis();
    }
    if (switchNumber == 5) {
      mainContainer.switchToLayout("compactcover");
      storeAttrib.setData("compactcover");
      fitVis();
    }
    if (switchNumber == 6) {
      mainContainer.switchToLayout("stick");
      storeAttrib.setData("stick");
      fitVis();
    }
  }
  complete;

  delete normalVisMenu;
}

// Don't switch from compactVis to normalVis:
compactVisPopup() {
  PopupMenu compactVisMenu;
  compactVisMenu = new PopupMenu;

  compactVisMenu.addCommand("Normal ", 1, (mainNormal.isVisible()), 0);
  compactVisMenu.addCommand("Normal Cover Art ", 2, (mainNormalCover.isVisible()), 0);
  compactVisMenu.addCommand("Compact ", 3, (mainCompact.isVisible()), 0);
  compactVisMenu.addCommand("Compact Cover Art ", 4, (mainCompactCover.isVisible()), 0);
  compactVisMenu.addCommand("Compact Visualization ", 5, (mainCompactVis.isVisible()), 0);
  compactVisMenu.addCommand("Stick ", 6, (mainStick.isVisible()), 0);

  if (compactVisMenu == Null) { return; }
  Int switchNumber = compactVisMenu.popAtMouse();
  if (switchNumber > 0) {
    if (switchNumber == 1) {
      mainContainer.switchToLayout("normal");
      storeAttrib.setData("normal");
      fitVis();
    }
    if (switchNumber == 2) {
      mainContainer.switchToLayout("normalcover");
      storeAttrib.setData("normalcover");
      fitVis();
    }
    if (switchNumber == 3) {
      mainContainer.switchToLayout("compact");
      storeAttrib.setData("compact");
      fitVis();
    }
    if (switchNumber == 4) {
      mainContainer.switchToLayout("compactcover");
      storeAttrib.setData("compactcover");
      fitVis();
    }
    if (switchNumber == 5) {
      mainContainer.switchToLayout("compactvis");
      visContainer.switchToLayout("compact");
      storeAttrib.setData("compactvis");
      fitVis();
    }
    if (switchNumber == 6) {
      mainContainer.switchToLayout("stick");
      storeAttrib.setData("stick");
      fitVis();
    }
  }
  complete;

  delete compactVisMenu;
}
