/* volanim-songtitle-thumb.m

FrisbeeMonkey's Volume by Frames adapted for SongTitle text objects and visible
volume sliders.

INSTRUCTIONS

1. Create a png image with frames for the animated volume bar.

2. Give the image a bitmap id and tell Winamp where to find it:

   <bitmap id="volume.anim" file="png/volanim.png"/>

3. Create a black to white gradient image for the volume map and tell Winamp
   where to find it:

   <bitmap id="volume.map" file="png/volmap.png"/>

4. Create two png's for the volume button, give them bitmap id's, and tell
   Winamp where to find them:

   <bitmap id="volume.thumb" file="player/volume-thumb.png"/>
   <bitmap id="volume.thumb.pressed" file="player/volume-thumb-pressed.png"/>

5. Call the script AFTER the following three XML objects:

  <Text
    id="SongTitle" display="SongTitle" ticker="1"
    x="?" y="?" w="?" h="?" fontsize="?" color="?"
  />

  <AnimatedLayer
    id="VolumeAnim"
    x="?" y="?" w="?" h="?"
    image="volume.anim"
    move="0" autoplay="0" autoreplay="0"
  />

  <slider
    id="Volume" action="VOLUME"
    x="?" y="?" w="?" h="?"
    thumb="volume.thumb"
    downThumb="volume.thumb.pressed"
  />

  <script id="volanim" file="scripts/volanim-songtitle-thumb.maki"/>

6. Copy this file and volanim-songtitle-thumb.maki into the scripts folder.

FrisbeeMonkey's Volume by Frames is archived here:
https://web.archive.org/web/20160311101829/http://frisbeemonkey.com/winamp3/references/maki.php?offset=20&sorter=name
*/

#include "../lib/std.mi"

Global AnimatedLayer VolumeAnim;
Global Slider VolumeSlider;
Global Map VolumeMap;
Global Text SongTitle;
Global Timer TickerTimer;
Global Boolean VolumeChanging;

Function setVolumeAnim(int volValue);
Function updateVolumeAnim(int x, int y);

System.onScriptLoaded() {
  Group ScriptGrp = getScriptGroup();

  VolumeAnim = ScriptGrp.findObject("VolumeAnimst");
  VolumeSlider = ScriptGrp.findObject("Volumest");
  SongTitle = ScriptGrp.findObject("SongTitlest");

  TickerTimer = new Timer;
  TickerTimer.setDelay(750);

  VolumeMap = new Map;
  VolumeMap.loadMap("mapstick.volume");

  setVolumeAnim(System.getVolume());
}

TickerTimer.onTimer() {
  SongTitle.setText("");
  TickerTimer.stop();
}

System.onVolumeChanged(int newVol) {
  setVolumeAnim(newVol);
  int p = (newVol * 100) / 255;
  TickerTimer.stop();
  TickerTimer.start();
  SongTitle.setAlternateText("Volume: " + System.integerToString(p) + "%");
}

setVolumeAnim(int Value) {
  int f = (Value * (VolumeAnim.getLength()-1)) / 255;
    if (Value > 0) {
    VolumeAnim.gotoFrame(f+1);
  }
  if (Value == 255) {
    VolumeAnim.gotoFrame(f);
  }
  if (Value == 0) {
    VolumeAnim.gotoFrame(0);
  }
}

VolumeSlider.onLeftButtonDown(int x, int y) {
  VolumeChanging = 1;
  updateVolumeAnim(x, y);
}

VolumeSlider.onLeftButtonUp(int x, int y) {
  if (VolumeChanging) {
    VolumeChanging = 0;
    updateVolumeAnim(x, y);
  }
}

VolumeSlider.onMouseMove(int x, int y) {
  if (VolumeChanging) {
    updateVolumeAnim(x, y);
  }
}

updateVolumeAnim(int x, int y) {
  int newValue = VolumeMap.getValue(x - VolumeAnim.getLeft(), y - VolumeAnim.getTop());
  System.setVolume(newValue);
}

System.onScriptUnloading() {
  delete TickerTimer;
  delete VolumeMap;
}
