/* volanim.m

Example of an Animated Volume bar using Frames
Simplified from FrisbeeMonkey's volumebyframe.m

Using this script

1. Add the following tags with identical "?" values to your XML:

   <AnimatedLayer
     id="VolumeAnim" image="volume.anim"
     x="?" y="?" w="?" h="?"
     move="0" autoplay="0" autoreplay="0"
   />
    <Slider
      id="HiddenVolume" action="VOLUME"
      x="?" y="?" w="?" h="?"
      thumb="hidden.thumb" downThumb="hidden.thumb"
      orientation="vertical"
      alpha="0"
    />

Change the position(x,y) and size(w,h) of "VolumeAnim" to the specifics
of your layer. (NOTE: w and h refer to just one frame, not the whole image)

2. Define your gradient map with your other elements using:
   <bitmap id="volume.map" file="player/volmap.png"/>

If you need help creating a map file, check:
https://web.archive.org/web/20050409195028/http://www.stefanweb.com/wa3/tutorials.html#UsingMaps

3. Copy this script (and volanim.maki) to your scripts folder.

4. If you don't have volanim.maki then compile it.

5. Add this line to the group that contains your layer:
   <script id="volanim" file="scripts/volanim.maki"/>

6. Refresh your skin (F5) and try it out. */

#include "../lib/std.mi"

Function setVolumeAnim(int volValue);
Function updateVolume(int x, int y);

Global AnimatedLayer Volume;
Global Map VolumeMap;
Global Boolean VolumeChanging;

System.onScriptLoaded() {
  Group ScriptGrp = getScriptGroup();
  Volume = ScriptGrp.findObject("VolumeAnim");
  VolumeMap = new Map;
  VolumeMap.loadMap("volume.map");
  setVolumeAnim(System.getVolume());
}

System.onVolumeChanged(int newvol) {
  setVolumeAnim(newvol);
}

setVolumeAnim(int volValue) {
  int f = (volValue * (Volume.getLength()-1)) / 255;
  if (volValue > 0) {
    Volume.gotoFrame(f);
  }
  if (volValue == 0) {
    Volume.gotoFrame(0);
  }
}

Volume.onLeftButtonDown(int x, int y) {
  VolumeChanging = 1;
  updateVolume(x, y);
}

Volume.onLeftButtonUp(int x, int y) {
  if (VolumeChanging) {
    VolumeChanging = 0;
    updateVolume(x, y);
  }
}

Volume.onMouseMove(int x, int y) {
  if (VolumeChanging) {
    updateVolume(x, y);
  }
}

updateVolume(int x, int y) {
  int newValue = VolumeMap.getValue(x - Volume.getLeft(), y - Volume.getTop());
  System.setVolume(newValue);
}

System.onScriptUnloading() {
  delete VolumeMap;
}
