//-----------------------------------------------------------------------------
// volume.m
//
// Example of an animated volume
// based on seek.m created by RhinoTrip
//
// created by FrisbeeMonkey
//-----------------------------------------------------------------------------

//                            USING THIS SCRIPT:
//*****************************************************************************
  1.  Define the following in your XML:

        <AnimatedLayer
          id="VolumeAnim"
          x="121" y="231"
          w="25" h="25"
          image="player.anim.volume"
          move="0"
          autoplay="0"
          autoreplay="0"
        />

        <slider id="HiddenVolume"
          ACTION="Volume" x="0" y="0" w="40" h="2"
          thumb="player.anim.volume"
          alpha="0"
        />
				<script id="animvolume" file="scripts/volume.maki"/>

  2.  Define your gradient map with your other elements using:
        <bitmap id="player.map.volume" file="player/player-map-volume.png"/>

  3.  Make sure your ticker is called "SongTicker" and is in the same group as
      "VolumeAnim" and "HiddenVolume"  If you don't have a ticker, add one now.
  4.  Copy this script (and volume.maki) to your scripts folder.
  5.  If you don't have volume.maki, compile this script.
  6.  Add this line to the group that contains your animated layer
        <script id="animvolume" file="scripts/volume.maki"/>



//      Change the position(x,y) and size(w,h) of "VolumeAnim" to the specifics
//      of your layer. (w and h refer to just one frame, not the whole image)
//		"HiddenVolume" is a dummy layer to get volume updates, no
//      changes are needed.
//      If you need help creating a map file, check:
//        http://www.stefanweb.com/wa3/tutorials.html#UsingMaps
//  7.  Refresh your skin(F5) and try it out.
//*****************************************************************************

// never forget to include std.mi
#include "../../../lib/std.mi"

//declares volume functions
Function updateVolume(int v);

//declares global variables
Global AnimatedLayer Volume;
Global Map volMap;
Global Timer SongTickerTimer;
Global Text SongTicker;
Global Slider HiddenVolume;
Global Int VolChanging, VolFrames, VolFlag, TickerExist;
Global Group pcGroup;

//When the script is loaded, do this
System.onScriptLoaded() {

	// Get the group that has the objects we want
	pcGroup = getScriptGroup();

	// Now that we have the group, get the objects in the group
	Volume = pcGroup.findObject("VolumeAnim");
	HiddenVolume = pcGroup.findObject("HiddenVolume");
	SongTicker = pcGroup.findObject("songticker");

	// Initialize our timer
	SongTickerTimer = new Timer;
	SongTickerTimer.setDelay(750);

	// Load the gradient map
	volMap = new Map;
	volMap.loadMap("player.map.volume");

	//initialize volume variables
	VolChanging = 0;
	VolFlag = 0;
    VolFrames = Volume.getLength();
    VolFrames = VolFrames - 1;

	// Set animated volume to current volume
	float u;
	u = System.getVolume();
	u = u / 255;
	u = u * VolFrames;
	Volume.gotoFrame(u);

}

// Clears text area
SongTickerTimer.onTimer() {
  SongTicker.setText("");
  SongTickerTimer.stop();
}

// Reads mouse drag on volume knob
Volume.onMouseMove(int x, int y) {
	if (VolChanging)  {
		x = x - Volume.getLeft();
		y = y - Volume.getTop();

		if (volMap.inRegion(x, y)) {
			float v = volMap.getValue(x, y);
			if (!VolFlag) {
				updateVolume(v);
				if (v > 245) {
					VolFlag = 2;
					updateVolume(255);
				}
				if (v < 10) {
					VolFlag = 1;
					updateVolume(0);
				}
			} else {
				if ((VolFlag == 2) && (v > 120) && (v < 252)) VolFlag = 0;
				if ((VolFlag == 1) && (v < 120) && (v > 2)) VolFlag = 0;
			}
        }
    }
}

// Reads mouse clicks on volume knob
Volume.onLeftButtonDown(int x, int y) {
	VolChanging = 1;
}

Volume.onLeftButtonUp(int x, int y) {
	if (VolChanging) {
		x = x - Volume.getLeft();
		y = y - Volume.getTop();

		if (volMap.inRegion(x, y)) {
			float v = volMap.getValue(x, y);
			if (!VolFlag) {
				updateVolume(v);
				if (v > 245) {
					VolFlag = 2;
					updateVolume(255);
				}
				if (v < 10) {
					VolFlag = 1;
					updateVolume(0);
				}
			} else {
				if ((VolFlag == 2) && (v > 120) && (v < 252)) VolFlag = 0;
				if ((VolFlag == 1) && (v < 120) && (v > 2)) VolFlag = 0;
			}
		}
		VolChanging = 0;
	}
}

// Updates Volume after drag/click
updateVolume(float v) {
	if (v >= 0) {
		float k = v / 255;
		int g = k * VolFrames;
		Volume.gotoFrame(g);
		System.setVolume(v);

		int p = k * 100;
		SongTickerTimer.stop();
		SongTickerTimer.start();
		SongTicker.setText("Vol " + System.integerToString(p) + "%");
     }
}

// Reads System Volume Changes
HiddenVolume.onPostedPosition(int p) {
	if (!VolChanging) {
		Float f = getVolume();
		f = f * VolFrames;
		f = f / 255;
		Volume.gotoFrame(f);
	}
}

System.onScriptUnloading() {
	delete SongTickerTimer;
	delete VolMap;
}